@@ -1,4 +1,22 @@
-require(reshape2)
+# Copyright 2010--2020 The Tor Project
+# See LICENSE for licensing information
+#
+# Be sure to run this file through lintr after making non-trivial changes:
+#
+#   require(ggplot2)
+#   require(lintr)
+#   lint("rserve-init.R")
+#
+# The following warnings can be ignored:
+#
+#  - style: Commented code should be removed. (Refers to code above.)
+#  - style: Variable and function name style should be snake_case. (Refers to
+#    FUN in write_data function.)
+#  - style: Variable and function names should not be longer than 30 characters.
+#    (Refers to prepare_ and plot_ functions for graphs with long names.)
+#  - warning: no visible binding for global variable ‘...’ (Refers to columns
+#    passed in aes() function.)
+
 require(ggplot2)
 require(RColorBrewer)
 require(scales)
@@ -319,6 +337,16 @@ formatter <- function(x, ...) {
   format(x, ..., scientific = FALSE, big.mark = " ")
 }
 
+# Helper function that takes a unit as input and returns a format function that
+# takes breaks as input and returns labels with that unit as output. In contrast
+# to the unit_format function in our somewhat outdated scales package this
+# function determines more reasonably how many digits it needs to print.
+custom_unit_format <- function(unit) {
+  function(x) {
+    paste(format(x, scientific = FALSE, big.mark = " "), unit)
+  }
+}
+
 theme_update(
   # Make plot title centered, and leave some room to the plot.
   plot.title = element_text(hjust = 0.5, margin = margin(b = 11)),
@@ -334,14 +362,15 @@ copyright_notice <- "The Tor Project - https://metrics.torproject.org/"
 
 stats_dir <- "/srv/metrics.torproject.org/metrics/shared/stats/"
 
-no_data_available_dir <- "/srv/metrics.torproject.org/metrics/src/main/R/rserver/"
+no_data_available_dir <-
+  "/srv/metrics.torproject.org/metrics/src/main/R/rserver/"
 
 # Helper function that copies the appropriate no data object to filename.
 copy_no_data <- function(filename) {
   len <- nchar(filename)
   extension <- substr(filename, len - 3, len)
   if (".csv" == extension) {
-    write("# No data available for the given parameters.", file=filename)
+    write("# No data available for the given parameters.", file = filename)
   } else {
     file.copy(paste(no_data_available_dir, "no-data-available", extension,
       sep = ""), filename)
@@ -406,7 +435,6 @@ prepare_versions <- function(start_p = NULL, end_p = NULL) {
 plot_versions <- function(start_p, end_p, path_p) {
   s <- prepare_versions(start_p, end_p)
   known_versions <- unique(s$version)
-  getPalette <- colorRampPalette(brewer.pal(12, "Paired"))
   colours <- data.frame(breaks = known_versions,
     values = rep(brewer.pal(min(12, length(known_versions)), "Paired"),
                  len = length(known_versions)),
@@ -437,7 +465,7 @@ prepare_platforms <- function(start_p = NULL, end_p = NULL) {
     filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     mutate(platform = tolower(platform)) %>%
-    dcast(date ~ platform, value.var = "relays")
+    spread(platform, relays)
 }
 
 plot_platforms <- function(start_p, end_p, path_p) {
@@ -468,32 +496,45 @@ prepare_dirbytes <- function(start_p = NULL, end_p = NULL) {
         bwread = col_skip(),
         bwwrite = col_skip(),
         dirread = col_double(),
-        dirwrite = col_double())) %>%
+        dirwrite = col_double(),
+        dirauthread = col_double(),
+        dirauthwrite = col_double())) %>%
     filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     filter(is.na(isexit)) %>%
     filter(is.na(isguard)) %>%
     mutate(dirread = dirread * 8 / 1e9,
-      dirwrite = dirwrite * 8 / 1e9) %>%
-    select(date, dirread, dirwrite)
+      dirwrite = dirwrite * 8 / 1e9,
+      dirauthread = dirauthread * 8 / 1e9,
+      dirauthwrite = dirauthwrite * 8 / 1e9) %>%
+    select(date, dirread, dirwrite, dirauthread, dirauthwrite)
 }
 
 plot_dirbytes <- function(start_p, end_p, path_p) {
   prepare_dirbytes(start_p, end_p) %>%
     gather(variable, value, -date) %>%
-    complete(date = full_seq(date, period = 1), nesting(variable)) %>%
-    ggplot(aes(x = date, y = value, colour = variable)) +
+    mutate(readwrite = ifelse(grepl("read", variable), "dirread", "dirwrite"),
+      authority = factor(
+          ifelse(grepl("auth", variable), "authorities", "mirrors"),
+          levels = c("authorities", "mirrors"))) %>%
+    complete(date = full_seq(date, period = 1),
+      nesting(readwrite, authority)) %>%
+    ggplot(aes(x = date, y = value, colour = readwrite)) +
     geom_line() +
+    facet_grid(authority ~ ., scales = "free_y", space = "free_y") +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "",
+      labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     scale_colour_hue(name = "",
         breaks = c("dirwrite", "dirread"),
         labels = c("Written dir bytes", "Read dir bytes")) +
     ggtitle("Number of bytes spent on answering directory requests") +
     labs(caption = copyright_notice) +
-    theme(legend.position = "top")
+    theme(legend.position = "top",
+          strip.text.y = element_text(angle = 0, hjust = 0),
+          strip.background = element_rect(fill = NA))
   ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
 }
 
@@ -526,7 +567,8 @@ plot_relayflags <- function(start_p, end_p, flag_p, path_p) {
 
 prepare_torperf <- function(start_p = NULL, end_p = NULL, server_p = NULL,
     filesize_p = NULL) {
-  read_csv(file = paste(stats_dir, "torperf-1.1.csv", sep = ""),
+  read_csv(file = paste(stats_dir, "onionperf-including-partials.csv",
+        sep = ""),
       col_types = cols(
         date = col_date(format = ""),
         filesize = col_double(),
@@ -534,10 +576,7 @@ prepare_torperf <- function(start_p = NULL, end_p = NULL, server_p = NULL,
         server = col_character(),
         q1 = col_double(),
         md = col_double(),
-        q3 = col_double(),
-        timeouts = col_skip(),
-        failures = col_skip(),
-        requests = col_skip())) %>%
+        q3 = col_double())) %>%
     filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     filter(if (!is.null(server_p)) server == server_p else TRUE) %>%
@@ -558,7 +597,7 @@ plot_torperf <- function(start_p, end_p, server_p, filesize_p, path_p) {
     geom_line(aes(colour = source), size = 0.75) +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "s"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "s"),
       limits = c(0, NA)) +
     scale_fill_hue(name = "Source") +
     scale_colour_hue(name = "Source") +
@@ -572,36 +611,26 @@ plot_torperf <- function(start_p, end_p, server_p, filesize_p, path_p) {
 }
 
 prepare_torperf_failures <- function(start_p = NULL, end_p = NULL,
-    server_p = NULL, filesize_p = NULL) {
-  read_csv(file = paste(stats_dir, "torperf-1.1.csv", sep = ""),
+    server_p = NULL) {
+  read_csv(file = paste(stats_dir, "onionperf-failures.csv", sep = ""),
       col_types = cols(
         date = col_date(format = ""),
-        filesize = col_double(),
         source = col_character(),
         server = col_character(),
-        q1 = col_skip(),
-        md = col_skip(),
-        q3 = col_skip(),
         timeouts = col_double(),
         failures = col_double(),
         requests = col_double())) %>%
     filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
-    filter(if (!is.null(filesize_p))
-        filesize == ifelse(filesize_p == "50kb", 50 * 1024,
-        ifelse(filesize_p == "1mb", 1024 * 1024, 5 * 1024 * 1024)) else
-        TRUE) %>%
     filter(if (!is.null(server_p)) server == server_p else TRUE) %>%
     filter(requests > 0) %>%
-    transmute(date, filesize, source, server, timeouts = timeouts / requests,
+    transmute(date, source, server, timeouts = timeouts / requests,
         failures = failures / requests)
 }
 
-plot_torperf_failures <- function(start_p, end_p, server_p, filesize_p,
-    path_p) {
-  prepare_torperf_failures(start_p, end_p, server_p, filesize_p) %>%
-    filter(source != "") %>%
-    gather(variable, value, -c(date, filesize, source, server)) %>%
+plot_torperf_failures <- function(start_p, end_p, server_p, path_p) {
+  prepare_torperf_failures(start_p, end_p, server_p) %>%
+    gather(variable, value, -c(date, source, server)) %>%
     mutate(variable = factor(variable, levels = c("timeouts", "failures"),
       labels = c("Timeouts", "Failures"))) %>%
     ggplot(aes(x = date, y = value, colour = source)) +
@@ -611,10 +640,7 @@ plot_torperf_failures <- function(start_p, end_p, server_p, filesize_p,
     scale_y_continuous(name = "", labels = percent, limits = c(0, NA)) +
     scale_colour_hue(name = "Source") +
     facet_grid(variable ~ .) +
-    ggtitle(paste("Timeouts and failures of",
-        ifelse(filesize_p == "50kb", "50 KiB",
-        ifelse(filesize_p == "1mb", "1 MiB", "5 MiB")),
-        "requests to", server_p, "server")) +
+    ggtitle(paste("Timeouts and failures of requests to", server_p, "server")) +
     labs(caption = copyright_notice) +
     theme(legend.position = "top")
   ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
@@ -646,7 +672,7 @@ plot_onionperf_buildtimes <- function(start_p, end_p, path_p) {
     facet_grid(position ~ .) +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "ms"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "ms"),
       limits = c(0, NA)) +
     scale_fill_hue(name = "Source") +
     scale_colour_hue(name = "Source") +
@@ -684,7 +710,7 @@ plot_onionperf_latencies <- function(start_p, end_p, server_p, path_p) {
     geom_line(aes(y = low, colour = source), size = 0.375) +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "ms"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "ms"),
       limits = c(0, NA)) +
     scale_fill_hue(name = "Source") +
     scale_colour_hue(name = "Source") +
@@ -724,7 +750,7 @@ plot_onionperf_throughput <- function(start_p, end_p, server_p, path_p) {
     geom_line(aes(y = low / 1000, colour = source), size = 0.375) +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Mbps"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "Mbps"),
       limits = c(0, NA)) +
     scale_fill_hue(name = "Source") +
     scale_colour_hue(name = "Source") +
@@ -748,7 +774,7 @@ prepare_connbidirect <- function(start_p = NULL, end_p = NULL) {
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     mutate(quantile = paste("X", quantile, sep = ""),
       fraction = fraction / 100) %>%
-    dcast(date + direction ~ quantile, value.var = "fraction") %>%
+    spread(quantile, fraction) %>%
     rename(q1 = X0.25, md = X0.5, q3 = X0.75)
 }
 
@@ -800,8 +826,7 @@ prepare_bandwidth_flags <- function(start_p = NULL, end_p = NULL) {
     filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
     filter(!is.na(have_exit_flag)) %>%
     filter(!is.na(have_guard_flag)) %>%
-    dcast(date + have_guard_flag + have_exit_flag ~ variable,
-      value.var = "value")
+    spread(variable, value)
 }
 
 plot_bandwidth_flags <- function(start_p, end_p, path_p) {
@@ -818,7 +843,7 @@ plot_bandwidth_flags <- function(start_p, end_p, path_p) {
     geom_area() +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     scale_fill_manual(name = "",
       values = c("#03B3FF", "#39FF02", "#FFFF00", "#AAAA99")) +
@@ -842,7 +867,7 @@ plot_bandwidth <- function(start_p, end_p, path_p) {
     geom_line() +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     scale_colour_hue(name = "", h.start = 90,
         breaks = c("advbw", "bwhist"),
@@ -1131,7 +1156,7 @@ prepare_advbwdist_perc <- function(start_p = NULL, end_p = NULL, p_p = NULL) {
     transmute(date, percentile = as.factor(percentile),
       variable = ifelse(is.na(isexit), "all", "exits"),
       advbw = advbw * 8 / 1e9) %>%
-    dcast(date + percentile ~ variable, value.var = "advbw") %>%
+    spread(variable, advbw) %>%
     rename(p = percentile)
 }
 
@@ -1146,7 +1171,8 @@ plot_advbwdist_perc <- function(start_p, end_p, p_p, path_p) {
     geom_line() +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "",
+      labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     scale_colour_hue(name = "Percentile") +
     ggtitle("Advertised bandwidth distribution") +
@@ -1169,7 +1195,7 @@ prepare_advbwdist_relay <- function(start_p = NULL, end_p = NULL, n_p = NULL) {
     transmute(date, relay = as.factor(relay),
       variable = ifelse(is.na(isexit), "all", "exits"),
       advbw = advbw * 8 / 1e9) %>%
-    dcast(date + relay ~ variable, value.var = "advbw") %>%
+    spread(variable, advbw) %>%
     rename(n = relay)
 }
 
@@ -1184,7 +1210,8 @@ plot_advbwdist_relay <- function(start_p, end_p, n_p, path_p) {
     geom_line() +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "",
+      labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     scale_colour_hue(name = "n") +
     ggtitle("Advertised bandwidth of n-th fastest relays") +
@@ -1221,6 +1248,35 @@ plot_hidserv_dir_onions_seen <- function(start_p, end_p, path_p) {
   ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
 }
 
+prepare_hidserv_dir_v3_onions_seen <- function(start_p = NULL, end_p = NULL) {
+  read_csv(file = paste(stats_dir, "v3hidserv.csv", sep = ""),
+      col_types = cols(
+        date = col_date(format = ""),
+        type = col_factor(levels = NULL),
+        wmean = col_skip(),
+        wmedian = col_skip(),
+        wiqm = col_double(),
+        frac = col_double(),
+        stats = col_skip())) %>%
+    filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
+    filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
+    filter(type == "dir-onions-seen") %>%
+    transmute(date, onions = ifelse(frac >= 0.01, wiqm, NA), frac)
+}
+
+plot_hidserv_dir_v3_onions_seen <- function(start_p, end_p, path_p) {
+  prepare_hidserv_dir_v3_onions_seen(start_p, end_p) %>%
+    complete(date = full_seq(date, period = 1)) %>%
+    ggplot(aes(x = date, y = onions)) +
+    geom_line() +
+    scale_x_date(name = "", breaks = custom_breaks,
+      labels = custom_labels, minor_breaks = custom_minor_breaks) +
+    scale_y_continuous(name = "", limits = c(0, NA), labels = formatter) +
+    ggtitle("Unique .onion v3 addresses") +
+    labs(caption = copyright_notice)
+  ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
+}
+
 prepare_hidserv_rend_relayed_cells <- function(start_p = NULL, end_p = NULL) {
   read_csv(file = paste(stats_dir, "hidserv.csv", sep = ""),
       col_types = cols(
@@ -1245,7 +1301,7 @@ plot_hidserv_rend_relayed_cells <- function(start_p, end_p, path_p) {
     geom_line() +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     ggtitle("Onion-service traffic") +
     labs(caption = copyright_notice)
@@ -1267,7 +1323,7 @@ prepare_webstats_tb <- function(start_p = NULL, end_p = NULL) {
     filter(request_type %in% c("tbid", "tbsd", "tbup", "tbur")) %>%
     group_by(log_date, request_type) %>%
     summarize(count = sum(count)) %>%
-    dcast(log_date ~ request_type, value.var = "count") %>%
+    spread(request_type, count) %>%
     rename(date = log_date, initial_downloads = tbid,
       signature_downloads = tbsd, update_pings = tbup,
       update_requests = tbur)
@@ -1312,7 +1368,7 @@ prepare_webstats_tb_platform <- function(start_p = NULL, end_p = NULL) {
     filter(request_type %in% c("tbid", "tbup")) %>%
     group_by(log_date, platform, request_type) %>%
     summarize(count = sum(count)) %>%
-    dcast(log_date + platform ~ request_type, value.var = "count") %>%
+    spread(request_type, count, fill = 0) %>%
     rename(date = log_date, initial_downloads = tbid, update_pings = tbup)
 }
 
@@ -1360,7 +1416,7 @@ prepare_webstats_tb_locale <- function(start_p = NULL, end_p = NULL) {
     group_by(date, locale, request_type) %>%
     summarize(count = sum(count)) %>%
     mutate(request_type = factor(request_type, levels = c("tbid", "tbup"))) %>%
-    dcast(date + locale ~ request_type, value.var = "count") %>%
+    spread(request_type, count, fill = 0) %>%
     rename(initial_downloads = tbid, update_pings = tbup)
 }
 
@@ -1415,7 +1471,7 @@ prepare_webstats_tb_channel <- function(start_p = NULL, end_p = NULL) {
     filter(channel %in% c("a", "r")) %>%
     group_by(log_date, channel, request_type) %>%
     summarize(count = sum(count)) %>%
-    dcast(log_date + channel ~ request_type, value.var = "count") %>%
+    spread(request_type, count) %>%
     rename(date = log_date, update_pings = tbup, update_requests = tbur)
 }
 
@@ -1461,8 +1517,7 @@ prepare_webstats_tm <- function(start_p = NULL, end_p = NULL) {
     group_by(log_date, request_type) %>%
     summarize(count = sum(count)) %>%
     mutate(request_type = factor(request_type, levels = c("tmid", "tmup"))) %>%
-    dcast(log_date ~ request_type, value.var = "count", drop = FALSE,
-      fill = 0) %>%
+    spread(request_type, count, drop = FALSE, fill = 0) %>%
     rename(date = log_date, initial_downloads = tmid, update_pings = tmup)
 }
 
@@ -1613,7 +1668,7 @@ plot_advbw_ipv6 <- function(start_p, end_p, path_p) {
     geom_line() +
     scale_x_date(name = "", breaks = custom_breaks,
       labels = custom_labels, minor_breaks = custom_minor_breaks) +
-    scale_y_continuous(name = "", labels = unit_format(unit = "Gbit/s"),
+    scale_y_continuous(name = "", labels = custom_unit_format(unit = "Gbit/s"),
       limits = c(0, NA)) +
     scale_colour_hue(name = "", h.start = 90,
       breaks = c("total", "total_guard", "total_exit", "reachable_guard",
@@ -1671,8 +1726,8 @@ write_userstats <- function(start, end, node, path) {
   end <- min(end, as.character(Sys.Date()))
   c <- read.csv(paste("/srv/metrics.torproject.org/metrics/shared/stats/",
                 "clients.csv", sep = ""), stringsAsFactors = FALSE)
-  c <- c[c$date >= start & c$date <= end & c$country != '' &
-         c$transport == '' & c$version == '' & c$node == node, ]
+  c <- c[c$date >= start & c$date <= end & c$country != "" &
+         c$transport == "" & c$version == "" & c$node == node, ]
   u <- data.frame(country = c$country, users = c$clients,
                   stringsAsFactors = FALSE)
   u <- u[!is.na(u$users), ]
@@ -1684,26 +1739,26 @@ write_userstats <- function(start, end, node, path) {
   u <- u[1:10, ]
   u <- data.frame(
     cc = as.character(u$country),
-    country = sub('the ', '', countrynames(as.character(u$country))),
+    country = sub("the ", "", countrynames(as.character(u$country))),
     abs = round(u$users),
     rel = sprintf("%.2f", round(100 * u$users / total, 2)))
   write.csv(u, path, quote = FALSE, row.names = FALSE)
 }
 
 write_userstats_relay <- function(start, end, path) {
-  write_userstats(start, end, 'relay', path)
+  write_userstats(start, end, "relay", path)
 }
 
 write_userstats_bridge <- function(start, end, path) {
-  write_userstats(start, end, 'bridge', path)
+  write_userstats(start, end, "bridge", path)
 }
 
 write_userstats_censorship_events <- function(start, end, path) {
   end <- min(end, as.character(Sys.Date()))
   c <- read.csv(paste("/srv/metrics.torproject.org/metrics/shared/stats/",
                 "clients.csv", sep = ""), stringsAsFactors = FALSE)
-  c <- c[c$date >= start & c$date <= end & c$country != '' &
-         c$transport == '' & c$version == '' & c$node == 'relay', ]
+  c <- c[c$date >= start & c$date <= end & c$country != "" &
+         c$transport == "" & c$version == "" & c$node == "relay", ]
   r <- data.frame(date = c$date, country = c$country,
                   upturn = ifelse(!is.na(c$upper) &
                                   c$clients > c$upper, 1, 0),
@@ -1715,9 +1770,64 @@ write_userstats_censorship_events <- function(start, end, path) {
   r <- r[order(r$downturn, r$upturn, decreasing = TRUE), ]
   r <- r[1:10, ]
   r <- data.frame(cc = r$country,
-    country = sub('the ', '', countrynames(as.character(r$country))),
+    country = sub("the ", "", countrynames(as.character(r$country))),
     downturns = r$downturn,
     upturns = r$upturn)
   write.csv(r, path, quote = FALSE, row.names = FALSE)
 }
 
+prepare_bridgedb_transport <- function(start_p = NULL, end_p = NULL) {
+  read_csv(file = paste(stats_dir, "bridgedb-stats.csv", sep = ""),
+      col_types = cols(
+        date = col_date(format = ""),
+        distributor = col_skip(),
+        transport = col_character(),
+        requests = col_double())) %>%
+    filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
+    filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
+    group_by(date, transport) %>%
+    summarize(requests = sum(requests)) %>%
+    arrange(date, transport)
+}
+
+plot_bridgedb_transport <- function(start_p, end_p, path_p) {
+  prepare_bridgedb_transport(start_p, end_p) %>%
+    complete(date = full_seq(date, period = 1), nesting(transport)) %>%
+    ggplot(aes(x = date, y = requests, colour = transport)) +
+    geom_line(na.rm = TRUE) +
+    scale_x_date(name = "", breaks = custom_breaks,
+      labels = custom_labels, minor_breaks = custom_minor_breaks) +
+    scale_y_continuous(name = "", labels = formatter, limits = c(0, NA)) +
+    scale_colour_hue(name = "") +
+    ggtitle("BridgeDB requests by requested transport") +
+    labs(caption = copyright_notice)
+  ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
+}
+
+prepare_bridgedb_distributor <- function(start_p = NULL, end_p = NULL) {
+  read_csv(file = paste(stats_dir, "bridgedb-stats.csv", sep = ""),
+      col_types = cols(
+        date = col_date(format = ""),
+        distributor = col_character(),
+        transport = col_skip(),
+        requests = col_double())) %>%
+    filter(if (!is.null(start_p)) date >= as.Date(start_p) else TRUE) %>%
+    filter(if (!is.null(end_p)) date <= as.Date(end_p) else TRUE) %>%
+    group_by(date, distributor) %>%
+    summarize(requests = sum(requests)) %>%
+    arrange(date, distributor)
+}
+
+plot_bridgedb_distributor <- function(start_p, end_p, path_p) {
+  prepare_bridgedb_distributor(start_p, end_p) %>%
+    complete(date = full_seq(date, period = 1), nesting(distributor)) %>%
+    ggplot(aes(x = date, y = requests, colour = distributor)) +
+    geom_line(na.rm = TRUE) +
+    scale_x_date(name = "", breaks = custom_breaks,
+      labels = custom_labels, minor_breaks = custom_minor_breaks) +
+    scale_y_continuous(name = "", labels = formatter, limits = c(0, NA)) +
+    scale_colour_hue(name = "") +
+    ggtitle("BridgeDB requests by distributor") +
+    labs(caption = copyright_notice)
+  ggsave(filename = path_p, width = 8, height = 5, dpi = 150)
+}
