@@ -1,4 +1,4 @@
-/* Copyright 2011--2018 The Tor Project
+/* Copyright 2011--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.stats.bwhist;
@@ -25,14 +25,11 @@ import java.sql.Timestamp;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
-import java.util.Calendar;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import java.util.SortedSet;
-import java.util.TimeZone;
 import java.util.TreeSet;
 
 /**
@@ -88,7 +85,7 @@ public final class RelayDescriptorDatabaseImporter {
    */
   private CallableStatement csH;
 
-  private static Logger log
+  private static final Logger logger
       = LoggerFactory.getLogger(RelayDescriptorDatabaseImporter.class);
 
   /**
@@ -136,26 +133,20 @@ public final class RelayDescriptorDatabaseImporter {
         this.psSs = conn.prepareStatement("SELECT fingerprint "
             + "FROM statusentry WHERE validafter = ?");
         this.psR = conn.prepareStatement("INSERT INTO statusentry "
-            + "(validafter, nickname, fingerprint, descriptor, "
-            + "published, address, orport, dirport, isauthority, "
-            + "isbadexit, isbaddirectory, isexit, isfast, isguard, "
-            + "ishsdir, isnamed, isstable, isrunning, isunnamed, "
-            + "isvalid, isv2dir, isv3dir, version, bandwidth, ports, "
-            + "rawdesc) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
-            + "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+            + "(validafter, fingerprint, isauthority, isexit, isguard, "
+            + "isrunning) VALUES (?, ?, ?, ?, ?, ?)");
         this.csH = conn.prepareCall("{call insert_bwhist(?, ?, ?, ?, ?, "
             + "?)}");
         this.psU = conn.prepareStatement("INSERT INTO scheduled_updates "
             + "(date) VALUES (?)");
         this.scheduledUpdates = new HashSet<>();
       } catch (SQLException e) {
-        log.warn("Could not connect to database or prepare statements.", e);
+        logger.warn("Could not connect to database or prepare statements.", e);
       }
     }
 
     /* Initialize date format, so that we can format timestamps. */
     this.dateTimeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
-    this.dateTimeFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
   }
 
   private void addDateToScheduledUpdates(long timestamp)
@@ -169,7 +160,7 @@ public final class RelayDescriptorDatabaseImporter {
           this.dateTimeFormat.format(timestamp).substring(0, 10)
           + " 00:00:00").getTime();
     } catch (ParseException e) {
-      log.warn("Internal parsing error.", e);
+      logger.warn("Internal parsing error.", e);
       return;
     }
     if (!this.scheduledUpdates.contains(dateMillis)) {
@@ -182,19 +173,15 @@ public final class RelayDescriptorDatabaseImporter {
   /**
    * Insert network status consensus entry into database.
    */
-  public void addStatusEntryContents(long validAfter, String nickname,
-      String fingerprint, String descriptor, long published,
-      String address, long orPort, long dirPort,
-      SortedSet<String> flags, String version, long bandwidth,
-      String ports, byte[] rawDescriptor) {
+  public void addStatusEntryContents(long validAfter, String fingerprint,
+      SortedSet<String> flags) {
     if (this.importIntoDatabase) {
       try {
         this.addDateToScheduledUpdates(validAfter);
-        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
         Timestamp validAfterTimestamp = new Timestamp(validAfter);
         if (lastCheckedStatusEntries != validAfter) {
           insertedStatusEntries.clear();
-          this.psSs.setTimestamp(1, validAfterTimestamp, cal);
+          this.psSs.setTimestamp(1, validAfterTimestamp);
           ResultSet rs = psSs.executeQuery();
           while (rs.next()) {
             String insertedFingerprint = rs.getString(1);
@@ -205,32 +192,12 @@ public final class RelayDescriptorDatabaseImporter {
         }
         if (!insertedStatusEntries.contains(fingerprint)) {
           this.psR.clearParameters();
-          this.psR.setTimestamp(1, validAfterTimestamp, cal);
-          this.psR.setString(2, nickname);
-          this.psR.setString(3, fingerprint);
-          this.psR.setString(4, descriptor);
-          this.psR.setTimestamp(5, new Timestamp(published), cal);
-          this.psR.setString(6, address);
-          this.psR.setLong(7, orPort);
-          this.psR.setLong(8, dirPort);
-          this.psR.setBoolean(9, flags.contains("Authority"));
-          this.psR.setBoolean(10, flags.contains("BadExit"));
-          this.psR.setBoolean(11, flags.contains("BadDirectory"));
-          this.psR.setBoolean(12, flags.contains("Exit"));
-          this.psR.setBoolean(13, flags.contains("Fast"));
-          this.psR.setBoolean(14, flags.contains("Guard"));
-          this.psR.setBoolean(15, flags.contains("HSDir"));
-          this.psR.setBoolean(16, flags.contains("Named"));
-          this.psR.setBoolean(17, flags.contains("Stable"));
-          this.psR.setBoolean(18, flags.contains("Running"));
-          this.psR.setBoolean(19, flags.contains("Unnamed"));
-          this.psR.setBoolean(20, flags.contains("Valid"));
-          this.psR.setBoolean(21, flags.contains("V2Dir"));
-          this.psR.setBoolean(22, flags.contains("V3Dir"));
-          this.psR.setString(23, version);
-          this.psR.setLong(24, bandwidth);
-          this.psR.setString(25, ports);
-          this.psR.setBytes(26, rawDescriptor);
+          this.psR.setTimestamp(1, validAfterTimestamp);
+          this.psR.setString(2, fingerprint);
+          this.psR.setBoolean(3, flags.contains("Authority"));
+          this.psR.setBoolean(4, flags.contains("Exit"));
+          this.psR.setBoolean(5, flags.contains("Guard"));
+          this.psR.setBoolean(6, flags.contains("Running"));
           this.psR.executeUpdate();
           rrsCount++;
           if (rrsCount % autoCommitCount == 0)  {
@@ -239,8 +206,8 @@ public final class RelayDescriptorDatabaseImporter {
           insertedStatusEntries.add(fingerprint);
         }
       } catch (SQLException e) {
-        log.warn("Could not add network status consensus entry. We won't make "
-            + "any further SQL requests in this execution.", e);
+        logger.warn("Could not add network status consensus entry. We won't "
+            + "make any further SQL requests in this execution.", e);
         this.importIntoDatabase = false;
       }
     }
@@ -337,7 +304,7 @@ public final class RelayDescriptorDatabaseImporter {
     for (String bandwidthHistoryString : bandwidthHistoryStrings) {
       String[] parts = bandwidthHistoryString.split(" ");
       if (parts.length != 6) {
-        log.debug("Bandwidth history line does not have expected "
+        logger.debug("Bandwidth history line does not have expected "
             + "number of elements. Ignoring this line.");
         continue;
       }
@@ -345,13 +312,13 @@ public final class RelayDescriptorDatabaseImporter {
       try {
         intervalLength = Long.parseLong(parts[3].substring(1));
       } catch (NumberFormatException e) {
-        log.debug("Bandwidth history line does not have valid interval length "
-            + "'{} {}'. Ignoring this line.", parts[3], parts[4]);
+        logger.debug("Bandwidth history line does not have valid interval "
+            + "length '{} {}'. Ignoring this line.", parts[3], parts[4]);
         continue;
       }
       String[] values = parts[5].split(",");
       if (intervalLength % 900L != 0L) {
-        log.debug("Bandwidth history line does not contain "
+        logger.debug("Bandwidth history line does not contain "
             + "multiples of 15-minute intervals. Ignoring this line.");
         continue;
       } else if (intervalLength != 900L) {
@@ -369,7 +336,7 @@ public final class RelayDescriptorDatabaseImporter {
           values = newValues;
           intervalLength = 900L;
         } catch (NumberFormatException e) {
-          log.debug("Number format exception while parsing "
+          logger.debug("Number format exception while parsing "
               + "bandwidth history line. Ignoring this line.");
           continue;
         }
@@ -383,15 +350,16 @@ public final class RelayDescriptorDatabaseImporter {
         dateStart = dateTimeFormat.parse(parts[1] + " 00:00:00")
             .getTime();
       } catch (ParseException e) {
-        log.debug("Parse exception while parsing timestamp in "
+        logger.debug("Parse exception while parsing timestamp in "
             + "bandwidth history line. Ignoring this line.");
         continue;
       }
       if (Math.abs(published - intervalEnd)
           > 7L * 24L * 60L * 60L * 1000L) {
-        log.debug("Extra-info descriptor publication time {} and last interval "
-            + "time {} in {} line differ by more than 7 days! Not adding this "
-            + "line!", dateTimeFormat.format(published), intervalEndTime, type);
+        logger.debug("Extra-info descriptor publication time {} and last "
+            + "interval time {} in {} line differ by more than 7 days! Not "
+            + "adding this line!", dateTimeFormat.format(published),
+            intervalEndTime, type);
         continue;
       }
       long currentIntervalEnd = intervalEnd;
@@ -417,7 +385,7 @@ public final class RelayDescriptorDatabaseImporter {
           currentIntervalEnd -= intervalLength * 1000L;
         }
       } catch (NumberFormatException e) {
-        log.debug("Number format exception while parsing "
+        logger.debug("Number format exception while parsing "
             + "bandwidth history line. Ignoring this line.");
         continue;
       }
@@ -465,7 +433,7 @@ public final class RelayDescriptorDatabaseImporter {
               this.csH.executeBatch();
             }
           } catch (SQLException | ParseException e) {
-            log.warn("Could not insert bandwidth "
+            logger.warn("Could not insert bandwidth "
                 + "history line into database.  We won't make any "
                 + "further SQL requests in this execution.", e);
             this.importIntoDatabase = false;
@@ -541,14 +509,7 @@ public final class RelayDescriptorDatabaseImporter {
     for (NetworkStatusEntry statusEntry
         : consensus.getStatusEntries().values()) {
       this.addStatusEntryContents(consensus.getValidAfterMillis(),
-          statusEntry.getNickname(),
-          statusEntry.getFingerprint().toLowerCase(),
-          statusEntry.getDescriptor().toLowerCase(),
-          statusEntry.getPublishedMillis(), statusEntry.getAddress(),
-          statusEntry.getOrPort(), statusEntry.getDirPort(),
-          statusEntry.getFlags(), statusEntry.getVersion(),
-          statusEntry.getBandwidth(), statusEntry.getPortList(),
-          statusEntry.getStatusEntryBytes());
+          statusEntry.getFingerprint().toLowerCase(), statusEntry.getFlags());
     }
   }
 
@@ -579,8 +540,8 @@ public final class RelayDescriptorDatabaseImporter {
   public void commit() {
 
     /* Log stats about imported descriptors. */
-    log.info("Finished importing relay descriptors: {} network status entries "
-        + "and {} bandwidth history elements", rrsCount, rhsCount);
+    logger.info("Finished importing relay descriptors: {} network status "
+        + "entries and {} bandwidth history elements", rrsCount, rhsCount);
 
     /* Insert scheduled updates a second time, just in case the refresh
      * run has started since inserting them the first time in which case
@@ -595,7 +556,7 @@ public final class RelayDescriptorDatabaseImporter {
           this.psU.execute();
         }
       } catch (SQLException e) {
-        log.warn("Could not add scheduled dates "
+        logger.warn("Could not add scheduled dates "
             + "for the next refresh run.", e);
       }
     }
@@ -607,7 +568,7 @@ public final class RelayDescriptorDatabaseImporter {
 
         this.conn.commit();
       } catch (SQLException e) {
-        log.warn("Could not commit final records to database", e);
+        logger.warn("Could not commit final records to database", e);
       }
     }
   }
@@ -623,33 +584,33 @@ public final class RelayDescriptorDatabaseImporter {
   List<String[]> queryBandwidth() throws SQLException {
     List<String[]> statistics = new ArrayList<>();
     String columns = "date, isexit, isguard, bwread, bwwrite, dirread, "
-        + "dirwrite";
+        + "dirwrite, dirauthread, dirauthwrite";
     statistics.add(columns.split(", "));
     Statement st = this.conn.createStatement();
-    Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"),
-        Locale.US);
     String queryString = "SELECT " + columns + " FROM stats_bandwidth";
     try (ResultSet rs = st.executeQuery(queryString)) {
       while (rs.next()) {
-        String[] outputLine = new String[7];
-        outputLine[0] = rs.getDate("date", calendar).toLocalDate().toString();
+        String[] outputLine = new String[9];
+        outputLine[0] = rs.getDate("date").toLocalDate().toString();
         outputLine[1] = getBooleanFromResultSet(rs, "isexit");
         outputLine[2] = getBooleanFromResultSet(rs, "isguard");
         outputLine[3] = getLongFromResultSet(rs, "bwread");
         outputLine[4] = getLongFromResultSet(rs, "bwwrite");
         outputLine[5] = getLongFromResultSet(rs, "dirread");
         outputLine[6] = getLongFromResultSet(rs, "dirwrite");
+        outputLine[7] = getLongFromResultSet(rs, "dirauthread");
+        outputLine[8] = getLongFromResultSet(rs, "dirauthwrite");
         statistics.add(outputLine);
       }
     }
     return statistics;
   }
 
-  /** Retrieve the <code>boolean</code> value of the designated column in the
-   * current row of the given <code>ResultSet</code> object and format it as a
-   * <code>String</code> object with <code>"t"</code> for <code>true</code> and
-   * <code>"f"</code> for <code>false</code>, or return <code>null</code> if the
-   * retrieved value was <code>NULL</code>. */
+  /** Retrieve the {@code boolean} value of the designated column in the
+   * current row of the given {@code ResultSet} object and format it as a
+   * {@code String} object with {@code "t"} for {@code true} and
+   * {@code "f"} for {@code false}, or return {@code null} if the
+   * retrieved value was {@code NULL}. */
   private static String getBooleanFromResultSet(ResultSet rs,
       String columnLabel) throws SQLException {
     boolean result = rs.getBoolean(columnLabel);
@@ -660,10 +621,10 @@ public final class RelayDescriptorDatabaseImporter {
     }
   }
 
-  /** Retrieve the <code>long</code> value of the designated column in the
-   * current row of the given <code>ResultSet</code> object and format it as a
-   * <code>String</code> object, or return <code>null</code> if the retrieved
-   * value was <code>NULL</code>. */
+  /** Retrieve the {@code long} value of the designated column in the
+   * current row of the given {@code ResultSet} object and format it as a
+   * {@code String} object, or return {@code null} if the retrieved
+   * value was {@code NULL}. */
   private static String getLongFromResultSet(ResultSet rs, String columnLabel)
       throws SQLException {
     long result = rs.getLong(columnLabel);
@@ -677,7 +638,7 @@ public final class RelayDescriptorDatabaseImporter {
     try {
       this.conn.close();
     } catch (SQLException e) {
-      log.warn("Could not close database connection.", e);
+      logger.warn("Could not close database connection.", e);
     }
   }
 }
