@@ -1,4 +1,4 @@
-/* Copyright 2013--2018 The Tor Project
+/* Copyright 2013--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.stats.clients;
@@ -23,7 +23,7 @@ import java.util.TreeMap;
 
 public class Main {
 
-  private static Logger log = LoggerFactory.getLogger(Main.class);
+  private static final Logger logger = LoggerFactory.getLogger(Main.class);
 
   private static Database database;
 
@@ -33,36 +33,36 @@ public class Main {
   /** Executes this data-processing module. */
   public static void main(String[] args) throws Exception {
 
-    log.info("Starting clients module.");
+    logger.info("Starting clients module.");
 
-    log.info("Connecting to database.");
+    logger.info("Connecting to database.");
     database = new Database();
 
-    log.info("Reading relay descriptors and importing relevant parts into the "
-        + "database.");
+    logger.info("Reading relay descriptors and importing relevant parts into "
+        + "the database.");
     parseRelayDescriptors();
 
-    log.info("Reading bridge descriptors and importing relevant parts into the "
-        + "database.");
+    logger.info("Reading bridge descriptors and importing relevant parts into "
+        + "the database.");
     parseBridgeDescriptors();
 
-    log.info("Processing newly imported data.");
+    logger.info("Processing newly imported data.");
     database.processImported();
     database.commit();
 
-    log.info("Querying aggregated statistics from the database.");
+    logger.info("Querying aggregated statistics from the database.");
     new Writer().write(new File(baseDir, "stats/userstats.csv").toPath(),
         database.queryEstimated());
     new Writer().write(new File(baseDir, "stats/userstats-combined.csv")
         .toPath(), database.queryCombined());
 
-    log.info("Disconnecting from database.");
+    logger.info("Disconnecting from database.");
     database.close();
 
-    log.info("Running detector.");
+    logger.info("Running detector.");
     new Detector().detect();
 
-    log.info("Terminating clients module.");
+    logger.info("Terminating clients module.");
   }
 
   private static final long ONE_HOUR_MILLIS = 60L * 60L * 1000L;
@@ -104,20 +104,22 @@ public class Main {
     long dirreqStatsEndMillis = descriptor.getDirreqStatsEndMillis();
     long dirreqStatsIntervalLengthMillis =
         descriptor.getDirreqStatsIntervalLength() * 1000L;
+    SortedMap<String, Integer> responses = descriptor.getDirreqV3Resp();
     SortedMap<String, Integer> requests = descriptor.getDirreqV3Reqs();
     BandwidthHistory dirreqWriteHistory =
         descriptor.getDirreqWriteHistory();
-    parseRelayDirreqV3Reqs(fingerprint, publishedMillis,
-        dirreqStatsEndMillis, dirreqStatsIntervalLengthMillis, requests);
+    parseRelayDirreqV3Resp(fingerprint, publishedMillis, dirreqStatsEndMillis,
+        dirreqStatsIntervalLengthMillis, responses, requests);
     parseRelayDirreqWriteHistory(fingerprint, publishedMillis,
         dirreqWriteHistory);
   }
 
-  private static void parseRelayDirreqV3Reqs(String fingerprint,
+  private static void parseRelayDirreqV3Resp(String fingerprint,
       long publishedMillis, long dirreqStatsEndMillis,
       long dirreqStatsIntervalLengthMillis,
+      SortedMap<String, Integer> responses,
       SortedMap<String, Integer> requests) throws SQLException {
-    if (requests == null
+    if (responses == null
         || publishedMillis - dirreqStatsEndMillis > ONE_WEEK_MILLIS
         || dirreqStatsIntervalLengthMillis != ONE_DAY_MILLIS) {
       /* Cut off all observations that are one week older than
@@ -129,25 +131,43 @@ public class Main {
         - dirreqStatsIntervalLengthMillis;
     long utcBreakMillis = (dirreqStatsEndMillis / ONE_DAY_MILLIS)
         * ONE_DAY_MILLIS;
-    for (int i = 0; i < 2; i++) {
-      long fromMillis = i == 0 ? statsStartMillis
-          : utcBreakMillis;
-      long toMillis = i == 0 ? utcBreakMillis : dirreqStatsEndMillis;
-      if (fromMillis >= toMillis) {
-        continue;
+    double resp = ((double) responses.get("ok")) - 4.0;
+    if (resp > 0.0) {
+      for (int i = 0; i < 2; i++) {
+        long fromMillis = i == 0 ? statsStartMillis : utcBreakMillis;
+        long toMillis = i == 0 ? utcBreakMillis : dirreqStatsEndMillis;
+        if (fromMillis >= toMillis) {
+          continue;
+        }
+        double intervalFraction = ((double) (toMillis - fromMillis))
+            / ((double) dirreqStatsIntervalLengthMillis);
+        double total = 0L;
+        SortedMap<String, Double> requestsCopy = new TreeMap<>();
+        if (null != requests) {
+          for (Map.Entry<String, Integer> e : requests.entrySet()) {
+            if (e.getValue() < 4.0) {
+              continue;
+            }
+            double frequency = ((double) e.getValue()) - 4.0;
+            requestsCopy.put(e.getKey(), frequency);
+            total += frequency;
+          }
+        }
+        /* If we're not told any requests, or at least none of them are greater
+         * than 4, put in a default that we'll attribute all responses to. */
+        if (requestsCopy.isEmpty()) {
+          requestsCopy.put("??", 4.0);
+          total = 4.0;
+        }
+        for (Map.Entry<String, Double> e : requestsCopy.entrySet()) {
+          String country = e.getKey();
+          double val = resp * intervalFraction * e.getValue() / total;
+          database.insertIntoImported(fingerprint, "relay", "responses",
+              country, "", "", fromMillis, toMillis, val);
+        }
+        database.insertIntoImported(fingerprint, "relay", "responses", "", "",
+            "", fromMillis, toMillis, resp * intervalFraction);
       }
-      double intervalFraction =  ((double) (toMillis - fromMillis))
-          / ((double) dirreqStatsIntervalLengthMillis);
-      double sum = 0L;
-      for (Map.Entry<String, Integer> e : requests.entrySet()) {
-        String country = e.getKey();
-        double reqs = ((double) e.getValue()) - 4.0;
-        sum += reqs;
-        database.insertIntoImported(fingerprint, "relay", "responses", country,
-            "", "", fromMillis, toMillis, reqs * intervalFraction);
-      }
-      database.insertIntoImported(fingerprint, "relay", "responses", "", "",
-          "", fromMillis, toMillis, sum * intervalFraction);
     }
   }
 
@@ -240,6 +260,7 @@ public class Main {
     parseBridgeDirreqV3Resp(fingerprint, publishedMillis,
         dirreqStatsEndMillis, dirreqStatsIntervalLengthMillis,
         descriptor.getDirreqV3Resp(),
+        descriptor.getDirreqV3Reqs(),
         descriptor.getBridgeIps(),
         descriptor.getBridgeIpTransports(),
         descriptor.getBridgeIpVersions());
@@ -252,6 +273,7 @@ public class Main {
       long publishedMillis, long dirreqStatsEndMillis,
       long dirreqStatsIntervalLengthMillis,
       SortedMap<String, Integer> responses,
+      SortedMap<String, Integer> requests,
       SortedMap<String, Integer> bridgeIps,
       SortedMap<String, Integer> bridgeIpTransports,
       SortedMap<String, Integer> bridgeIpVersions) throws SQLException {
@@ -281,7 +303,8 @@ public class Main {
         database.insertIntoImported(fingerprint, "bridge", "responses", "", "",
             "", fromMillis, toMillis, resp * intervalFraction);
         parseBridgeRespByCategory(fingerprint, fromMillis, toMillis, resp,
-            dirreqStatsIntervalLengthMillis, "country", bridgeIps);
+            dirreqStatsIntervalLengthMillis, "country",
+            null != requests ? requests : bridgeIps);
         parseBridgeRespByCategory(fingerprint, fromMillis, toMillis, resp,
             dirreqStatsIntervalLengthMillis, "transport",
             bridgeIpTransports);
@@ -311,7 +334,7 @@ public class Main {
     /* If we're not told any frequencies, or at least none of them are
      * greater than 4, put in a default that we'll attribute all responses
      * to. */
-    if (total == 0) {
+    if (frequenciesCopy.isEmpty()) {
       switch (category) {
         case "country":
           frequenciesCopy.put("??", 4.0);
