@@ -1,4 +1,4 @@
-/* Copyright 2016--2018 The Tor Project
+/* Copyright 2016--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.stats.hidserv;
@@ -26,7 +26,8 @@ import java.util.TreeSet;
  * interface to a file and later to retrieve them. */
 public class DocumentStore<T extends Document> {
 
-  private static Logger log = LoggerFactory.getLogger(DocumentStore.class);
+  private static final Logger logger
+      = LoggerFactory.getLogger(DocumentStore.class);
 
   /** Document class, needed to create new instances when retrieving
    * documents. */
@@ -47,7 +48,7 @@ public class DocumentStore<T extends Document> {
     /* Retrieve existing documents. */
     Set<T> retrievedDocuments = this.retrieve(documentFile);
     if (retrievedDocuments == null) {
-      log.warn("Unable to read and update {}. Not storing documents.",
+      logger.warn("Unable to read and update {}. Not storing documents.",
           documentFile.getAbsoluteFile());
       return false;
     }
@@ -68,7 +69,7 @@ public class DocumentStore<T extends Document> {
     File documentTempFile = new File(documentFile.getAbsoluteFile()
         + ".tmp");
     if (documentTempFile.exists()) {
-      log.warn("Temporary document file {} still exists, "
+      logger.warn("Temporary document file {} still exists, "
           + "indicating that a previous execution did not terminate "
           + "cleanly.  Not storing documents.",
           documentTempFile.getAbsoluteFile());
@@ -90,7 +91,7 @@ public class DocumentStore<T extends Document> {
       documentFile.delete();
       documentTempFile.renameTo(documentFile);
     } catch (IOException e) {
-      log.warn("Unable to write {}. Not storing documents.",
+      logger.warn("Unable to write {}. Not storing documents.",
           documentFile.getAbsolutePath(), e);
       return false;
     }
@@ -125,7 +126,7 @@ public class DocumentStore<T extends Document> {
         if (!line.startsWith(" ")) {
           formattedString0 = line;
         } else if (formattedString0 == null) {
-          log.warn("First line in {} must not start with a space. Not "
+          logger.warn("First line in {} must not start with a space. Not "
               + "retrieving any previously stored documents.",
               documentFile.getAbsolutePath());
           return null;
@@ -140,7 +141,7 @@ public class DocumentStore<T extends Document> {
           T document = this.clazz.getDeclaredConstructor().newInstance();
           if (!document.parse(new String[] { formattedString0,
               line.substring(1) })) {
-            log.warn("Unable to read line {} from {}. Not retrieving any "
+            logger.warn("Unable to read line {} from {}. Not retrieving any "
                 + "previously stored documents.", lnr.getLineNumber(),
                 documentFile.getAbsolutePath());
             return null;
@@ -149,12 +150,12 @@ public class DocumentStore<T extends Document> {
         }
       }
     } catch (IOException e) {
-      log.warn("Unable to read {}. Not retrieving any previously stored "
+      logger.warn("Unable to read {}. Not retrieving any previously stored "
           + "documents.", documentFile.getAbsolutePath(), e);
       return null;
     } catch (InstantiationException | IllegalAccessException
         | NoSuchMethodException | InvocationTargetException e) {
-      log.warn("Unable to read {}. Cannot instantiate document object.",
+      logger.warn("Unable to read {}. Cannot instantiate document object.",
           documentFile.getAbsolutePath(), e);
       return null;
     }
