@@ -1,4 +1,4 @@
-/* Copyright 2016--2018 The Tor Project
+/* Copyright 2016--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.stats.hidserv;
@@ -20,7 +20,8 @@ import java.util.TreeSet;
  * observed by the relay. */
 public class Extrapolator {
 
-  private static Logger log = LoggerFactory.getLogger(Extrapolator.class);
+  private static final Logger logger
+      = LoggerFactory.getLogger(Extrapolator.class);
 
   /** Document file containing previously parsed reported hidden-service
    * statistics. */
@@ -47,14 +48,33 @@ public class Extrapolator {
   private DocumentStore<ExtrapolatedHidServStats>
       extrapolatedHidServStatsStore;
 
+  /** Document file containing previously parsed reported directory-requests
+   * statistics. */
+  private File reportedV3HidServStatsFile;
+
+  /** Document store for storing and retrieving reported directory-requests
+   * statistics. */
+  private DocumentStore<ReportedHidServStats> reportedV3HidServStatsStore;
+
+  /** Document file containing extrapolated directory-requests statistics. */
+  private File extrapolatedV3HidServStatsFile;
+
+  /** Document store for storing and retrieving extrapolated directory-requests
+   * statistics. */
+  private DocumentStore<ExtrapolatedHidServStats>
+      extrapolatedV3HidServStatsStore;
+
   /** Initializes a new extrapolator object using the given directory and
    * document stores. */
   public Extrapolator(File statusDirectory,
       DocumentStore<ReportedHidServStats> reportedHidServStatsStore,
+      DocumentStore<ReportedHidServStats> reportedV3HidServStatsStore,
       DocumentStore<ComputedNetworkFractions>
       computedNetworkFractionsStore,
       DocumentStore<ExtrapolatedHidServStats>
-      extrapolatedHidServStatsStore) {
+      extrapolatedHidServStatsStore,
+      DocumentStore<ExtrapolatedHidServStats>
+      extrapolatedV3HidServStatsStore) {
 
     /* Create File instances for the files and directories in the provided
      * status directory. */
@@ -64,11 +84,16 @@ public class Extrapolator {
         new File(statusDirectory, "computed-network-fractions");
     this.extrapolatedHidServStatsFile = new File(statusDirectory,
         "extrapolated-hidserv-stats");
-
+    this.reportedV3HidServStatsFile = new File(statusDirectory,
+        "reported-v3hidserv-stats");
+    this.extrapolatedV3HidServStatsFile = new File(statusDirectory,
+        "extrapolated-v3hidserv-stats");
     /* Store references to the provided document stores. */
     this.reportedHidServStatsStore = reportedHidServStatsStore;
     this.computedNetworkFractionsStore = computedNetworkFractionsStore;
     this.extrapolatedHidServStatsStore = extrapolatedHidServStatsStore;
+    this.reportedV3HidServStatsStore = reportedV3HidServStatsStore;
+    this.extrapolatedV3HidServStatsStore = extrapolatedV3HidServStatsStore;
   }
 
   /** Iterates over all reported stats and extrapolate network totals for
@@ -87,11 +112,48 @@ public class Extrapolator {
         this.reportedHidServStatsStore.retrieve(
         this.reportedHidServStatsFile);
 
+    Set<ExtrapolatedHidServStats> computedStats = computeExtrapolatedStats(
+        extrapolatedStats, reportedStats, "v2");
+
+    /* Store all extrapolated network totals to disk with help of the
+     * document store. */
+    return this.extrapolatedHidServStatsStore.store(
+        this.extrapolatedHidServStatsFile, computedStats);
+  }
+
+  /** Iterates over all reported stats and extrapolate network totals for
+   * those that have not been extrapolated before. */
+  public boolean extrapolateV3HidServStats() {
+
+    /* Retrieve previously extrapolated stats to avoid extrapolating them
+     * again. */
+    Set<ExtrapolatedHidServStats> extrapolatedStats =
+        this.extrapolatedV3HidServStatsStore.retrieve(
+        this.extrapolatedV3HidServStatsFile);
+
+    /* Retrieve all reported stats, even including those that have already
+     * been extrapolated. */
+    Set<ReportedHidServStats> reportedStats =
+        this.reportedV3HidServStatsStore.retrieve(
+        this.reportedV3HidServStatsFile);
+
+    Set<ExtrapolatedHidServStats> computedStats = computeExtrapolatedStats(
+        extrapolatedStats, reportedStats, "v3");
+
+    /* Store all extrapolated network totals to disk with help of the
+     * document store. */
+    return this.extrapolatedV3HidServStatsStore.store(
+        this.extrapolatedV3HidServStatsFile, computedStats);
+  }
+
+  private Set<ExtrapolatedHidServStats> computeExtrapolatedStats(
+      Set<ExtrapolatedHidServStats> extrapolatedStats,
+      Set<ReportedHidServStats> reportedStats, String version) {
     /* Make sure that all documents could be retrieved correctly. */
     if (extrapolatedStats == null || reportedStats == null) {
-      log.warn("Could not read previously parsed or extrapolated "
-          + "hidserv-stats. Skipping.");
-      return false;
+      logger.warn("Could not read previously parsed or extrapolated "
+          + "stats files. Skipping.");
+      return null;
     }
 
     /* Re-arrange reported stats by fingerprint. */
@@ -197,59 +259,67 @@ public class Extrapolator {
           }
         }
 
-        /* If we don't know a single consensus with valid-after time in
-         * the statistics interval, skip this stat. */
-        if (consensuses == 0) {
-          continue;
+        /* Compute means of network fractions, or assume 0.0 if we don't
+         * know a single consensus with valid-after time in the statistics
+         * interval. */
+        double fractionRendRelayedCells = consensuses == 0 ? 0.0
+            : sumFractionRendRelayedCells / consensuses;
+        double fractionDirOnionsSeen = consensuses == 0 ? 0.0
+            : sumFractionDirOnionsSeen / consensuses;
+
+        /* Extrapolate network totals. If we don't know a single
+         * consensus, store an empty statistic anyway to avoid processing
+         * these reported statistics over and over. */
+        ExtrapolatedHidServStats extrapolated =
+            new ExtrapolatedHidServStats(
+            statsDateMillis, fingerprint);
+        if (fractionRendRelayedCells > 0.0) {
+          extrapolated.setFractionRendRelayedCells(
+              fractionRendRelayedCells);
+          /* Extrapolating cells on rendezvous circuits is as easy as
+           * dividing the reported number by the computed network
+           * fraction. */
+          double extrapolatedRendRelayedCells =
+              stats.getRendRelayedCells() / fractionRendRelayedCells;
+          extrapolated.setExtrapolatedRendRelayedCells(
+              extrapolatedRendRelayedCells);
         }
-
-        /* Compute means of network fractions. */
-        double fractionRendRelayedCells =
-            sumFractionRendRelayedCells / consensuses;
-        double fractionDirOnionsSeen =
-            sumFractionDirOnionsSeen / consensuses;
-
-        /* If at least one fraction is positive, extrapolate network
-         * totals. */
-        if (fractionRendRelayedCells > 0.0
-            || fractionDirOnionsSeen > 0.0) {
-          ExtrapolatedHidServStats extrapolated =
-              new ExtrapolatedHidServStats(
-              statsDateMillis, fingerprint);
-          if (fractionRendRelayedCells > 0.0) {
-            extrapolated.setFractionRendRelayedCells(
-                fractionRendRelayedCells);
-            /* Extrapolating cells on rendezvous circuits is as easy as
-             * dividing the reported number by the computed network
-             * fraction. */
-            double extrapolatedRendRelayedCells =
-                stats.getRendRelayedCells() / fractionRendRelayedCells;
-            extrapolated.setExtrapolatedRendRelayedCells(
-                extrapolatedRendRelayedCells);
-          }
-          if (fractionDirOnionsSeen > 0.0) {
-            extrapolated.setFractionDirOnionsSeen(
-                fractionDirOnionsSeen);
+        if (fractionDirOnionsSeen > 0.0) {
+          extrapolated.setFractionDirOnionsSeen(
+              fractionDirOnionsSeen);
+          double extrapolatedDirOnionsSeen = 0;
+          if (version == "v2") {
             /* Extrapolating reported unique .onion addresses to the
              * total number in the network is more difficult.  In short,
              * each descriptor is stored to 12 (likely) different
              * directories, so we'll have to divide the reported number by
              * 12 and then by the computed network fraction of this
              * directory. */
-            double extrapolatedDirOnionsSeen =
+            extrapolatedDirOnionsSeen =
                 stats.getDirOnionsSeen() / (12.0 * fractionDirOnionsSeen);
-            extrapolated.setExtrapolatedDirOnionsSeen(
-                extrapolatedDirOnionsSeen);
+          } else if (version == "v3") {
+            /* When it comes to the number of onion services, the logic is
+             * identical between v2 and v3 stats but in the Extrapolator we
+             * should be dividing by 24 instead of 12. That's because in v3 onions,
+             * each descriptor has 2 replicas and 4 uploads for each replica, so
+             * in total each descriptor get uploaded to 8 HSDirs. Furthermore,
+             * each onion service maintains two descriptors at any given time
+             * (so 16 uploads), and rotates one descriptor at around 00:00UTC
+             * (so another 8 uploads). This means that between the interval of
+             * 12:00UTC to 12:00UTC the HSDirs will be exposed to 24 uploads for
+             * each HS. This is precisely the reason we choose to collect stats
+             * from 12:00UTC to 12:00UTC, and this logic has not been documented
+             * somewhere yet but we should find a good place to document it.
+             */
+             extrapolatedDirOnionsSeen =
+                 stats.getDirOnionsSeen() / (24.0 * fractionDirOnionsSeen);
           }
-          extrapolatedStats.add(extrapolated);
+          extrapolated.setExtrapolatedDirOnionsSeen(
+              extrapolatedDirOnionsSeen);
         }
+        extrapolatedStats.add(extrapolated);
       }
     }
-
-    /* Store all extrapolated network totals to disk with help of the
-     * document store. */
-    return this.extrapolatedHidServStatsStore.store(
-        this.extrapolatedHidServStatsFile, extrapolatedStats);
+    return extrapolatedStats;
   }
 }
-
