@@ -1,4 +1,4 @@
-/* Copyright 2016--2017 The Tor Project
+/* Copyright 2016--2020 The Tor Project
  * See LICENSE for licensing information */
 
 package org.torproject.metrics.web;
@@ -13,11 +13,6 @@ public class SourcesServlet extends AnyServlet {
 
   private static final long serialVersionUID = 6850919895248020945L;
 
-  @Override
-  public void init() throws ServletException {
-    super.init();
-  }
-
   @Override
   public void doGet(HttpServletRequest request,
       HttpServletResponse response) throws IOException, ServletException {
