@@ -43,6 +43,9 @@ define([
                 if (flag == "Stable") {
                     output.push([flag, "stable", "This relay is suitable for long-lived circuits."]);
                 }
+                if (flag == "StaleDesc") {
+                    output.push([flag, "staledesc", "This relay should upload a new descriptor because the old one is too old."]);
+                }
                 if (flag == "V2Dir") {
                     output.push([flag, "v2dir", "This relay implements the v2 directory protocol or higher."]);
                 }
@@ -237,10 +240,11 @@ define([
                     relay.as = relay.as ? relay.as : null;
                     relay.as_name = relay.as_name ? relay.as_name : null;
                     relay.transports = relay.transports ? relay.transports : null;
+                    relay.bridgedb_distributor = relay.bridgedb_distributor ? relay.bridgedb_distributor : null;
                     relay.fingerprint = relay.hashed_fingerprint ? relay.hashed_fingerprint : relay.fingerprint;
                     model.set({badexit: false});
                     relay.flags = model.parseflags(relay.flags, relay.is_bridge);
-
+                    relay.overload_general_timestamp = relay.overload_general_timestamp ? relay.overload_general_timestamp : null;
                     relay.version_consistent = relay.version == relay.platform.split(" ")[1];
                     if (relay.version_consistent) {
                         relay.version_status = relay.version_status ? relay.version_status : "recommended";
