@@ -1,4 +1,4 @@
--- Copyright 2017--2018 The Tor Project
+-- Copyright 2017--2020 The Tor Project
 -- See LICENSE for licensing information
 
 CREATE TABLE IF NOT EXISTS measurements (
@@ -17,6 +17,15 @@ CREATE TABLE IF NOT EXISTS measurements (
   writebytes INTEGER,
   readbytes INTEGER,
   didtimeout BOOLEAN,
+  partial10240 INTEGER,
+  partial20480 INTEGER,
+  partial51200 INTEGER,
+  partial102400 INTEGER,
+  partial204800 INTEGER,
+  partial512000 INTEGER,
+  partial1048576 INTEGER,
+  partial2097152 INTEGER,
+  partial5242880 INTEGER,
   dataperc0 INTEGER,
   dataperc10 INTEGER,
   dataperc20 INTEGER,
@@ -53,26 +62,59 @@ CREATE TABLE IF NOT EXISTS buildtimes (
 
 CREATE TYPE server AS ENUM ('public', 'onion');
 
-CREATE OR REPLACE VIEW onionperf AS
+CREATE OR REPLACE VIEW onionperf_including_partials AS
+WITH measurements_including_partials AS (
+  SELECT start,
+    filesize,
+    source,
+    endpointremote,
+    datacomplete
+  FROM measurements
+  UNION
+  SELECT start,
+    51200 AS filesize,
+    source,
+    endpointremote,
+    partial51200 AS datacomplete
+  FROM measurements
+  WHERE filesize > 51200
+  AND partial51200 IS NOT NULL
+  UNION
+  SELECT start,
+    1048576 AS filesize,
+    source,
+    endpointremote,
+    partial1048576 AS datacomplete
+  FROM measurements
+  WHERE filesize > 1048576
+  AND partial1048576 IS NOT NULL
+), grouped AS (
+  SELECT DATE(start) AS date,
+    filesize,
+    source,
+    CASE WHEN endpointremote LIKE '%.onion:%' THEN 'onion'
+      ELSE 'public' END AS server,
+    CASE WHEN COUNT(*) > 0 THEN
+      PERCENTILE_CONT(ARRAY[0.25,0.5,0.75]) WITHIN GROUP(ORDER BY datacomplete)
+      ELSE NULL END AS q
+  FROM measurements_including_partials
+  GROUP BY date, filesize, source, server
+)
 SELECT date,
   filesize,
   source,
   server,
-  CASE WHEN q IS NULL THEN NULL ELSE q[1] END AS q1,
-  CASE WHEN q IS NULL THEN NULL ELSE q[2] END AS md,
-  CASE WHEN q IS NULL THEN NULL ELSE q[3] END AS q3,
-  timeouts,
-  failures,
-  requests
-FROM (
+  CASE WHEN q IS NULL THEN NULL ELSE TRUNC(q[1]) END AS q1,
+  CASE WHEN q IS NULL THEN NULL ELSE TRUNC(q[2]) END AS md,
+  CASE WHEN q IS NULL THEN NULL ELSE TRUNC(q[3]) END AS q3
+FROM grouped
+ORDER BY date, filesize, source, server;
+
+CREATE OR REPLACE VIEW onionperf_failures AS
 SELECT DATE(start) AS date,
-  filesize,
   source,
   CASE WHEN endpointremote LIKE '%.onion:%' THEN 'onion'
     ELSE 'public' END AS server,
-  CASE WHEN COUNT(*) > 0 THEN
-    PERCENTILE_CONT(ARRAY[0.25,0.5,0.75]) WITHIN GROUP(ORDER BY datacomplete)
-    ELSE NULL END AS q,
   COUNT(CASE WHEN didtimeout OR datacomplete < 1 THEN 1 ELSE NULL END)
     AS timeouts,
   COUNT(CASE WHEN NOT didtimeout AND datacomplete >= 1
@@ -80,27 +122,8 @@ SELECT DATE(start) AS date,
   COUNT(*) AS requests
 FROM measurements
 WHERE DATE(start) < current_date - 1
-AND (endpointremote IS NULL OR endpointremote NOT SIMILAR TO '_{56}.onion%')
-GROUP BY date, filesize, source, server
-UNION
-SELECT DATE(start) AS date,
-  filesize,
-  '' AS source,
-  CASE WHEN endpointremote LIKE '%.onion:%' THEN 'onion'
-    ELSE 'public' END AS server,
-  CASE WHEN COUNT(*) > 0 THEN
-    PERCENTILE_CONT(ARRAY[0.25,0.5,0.75]) WITHIN GROUP(ORDER BY datacomplete)
-    ELSE NULL END AS q,
-  COUNT(CASE WHEN didtimeout OR datacomplete < 1 THEN 1 ELSE NULL END)
-    AS timeouts,
-  COUNT(CASE WHEN NOT didtimeout AND datacomplete >= 1
-    AND readbytes < filesize THEN 1 ELSE NULL END) AS failures,
-  COUNT(*) AS requests
-FROM measurements
-WHERE DATE(start) < current_date - 1
-AND (endpointremote IS NULL OR endpointremote NOT SIMILAR TO '_{56}.onion%')
-GROUP BY date, filesize, 3, server) sub
-ORDER BY date, filesize, source, server;
+GROUP BY date, source, server
+ORDER BY date, source, server;
 
 CREATE OR REPLACE VIEW buildtimes_stats AS
 SELECT date,
@@ -140,7 +163,6 @@ WITH filtered_measurements AS (
   WHERE DATE(start) < current_date - 1
   AND datarequest > 0
   AND dataresponse > 0
-  AND (endpointremote IS NULL OR endpointremote NOT SIMILAR TO '_{56}.onion%')
 ), quartiles AS (
   SELECT date,
     source,
@@ -172,6 +194,10 @@ ORDER BY date, source, server;
 -- downloading 1 bit, which happens to be the same value as the number of
 -- seconds for downloading 1 kilobit. We want the reciprocal of that value which
 -- has the unit kilobits per second.
+-- For 5MiB transfers we extract the number of seconds that have elapsed between
+-- receiving bytes 4,194,304 receiving bytes 4,194,304 and 5,242,880, which is
+-- a total amount of 1,048,576 bytes or 8,388,608 bits.
+
 CREATE OR REPLACE VIEW throughput_stats AS
 WITH filtered_measurements AS (
   SELECT DATE(start) AS date,
@@ -180,12 +206,11 @@ WITH filtered_measurements AS (
       ELSE 'public' END AS server,
     CASE WHEN filesize = 1048576 AND dataperc100 > dataperc50
       THEN 4194304 / (dataperc100 - dataperc50)
-      WHEN filesize = 5242880 AND dataperc20 > dataperc10
-      THEN 4194304 / (dataperc20 - dataperc10)
+      WHEN filesize = 5242880 AND dataperc100 > dataperc80
+      THEN 8388608 / (dataperc100 - dataperc80)
       ELSE NULL END AS kbps
   FROM measurements
   WHERE DATE(start) < current_date - 1
-  AND (endpointremote IS NULL OR endpointremote NOT SIMILAR TO '_{56}.onion%')
 ), quartiles AS (
   SELECT date,
     source,
